@@ -12,10 +12,10 @@ use POE::Component::IRC;
 use constant IRCNAME   => "wanna";
 use constant ALIASNAME => "pony";
 
-$Bot::BasicBot::VERSION=0.04;
+$Bot::BasicBot::VERSION = 0.05;
 
 use vars qw(@ISA @EXPORT);
-@ISA = qw(Exporter);
+@ISA    = qw(Exporter);
 @EXPORT = qw(say emote);
 
 =head1 NAME
@@ -64,24 +64,20 @@ with the value supplied.
 
 =cut
 
-sub new
-{
+sub new {
     my $class = shift;
-    my $this = bless {}, $class;
+    my $this  = bless {}, $class;
 
     # call the set methods
     my %args = @_;
-    foreach my $method (keys %args)
-    {
-      if ($this->can($method))
-      {
-        $this->$method($args{$method})
-      }
-      else
-      {
-        $this->{$method} = $args{$method};
-        #croak "Invalid argument '$method'";
-      }
+    foreach my $method ( keys %args ) {
+        if ( $this->can($method) ) {
+            $this->$method( $args{$method} );
+        } else {
+            $this->{$method} = $args{$method};
+
+            #croak "Invalid argument '$method'";
+        }
     }
 
     return $this;
@@ -93,40 +89,41 @@ Runs the bot.  Hands the control over to the POE core.
 
 =cut
 
-sub run
-{
-  my $this = shift;
+sub run {
+    my $this = shift;
 
-  # yep, we use irc
-  POE::Component::IRC->new( IRCNAME ) or
-      die "Can't instantiate new IRC component!\n";
+    # yep, we use irc
+    POE::Component::IRC->new(IRCNAME)
+      or die "Can't instantiate new IRC component!\n";
 
-  # create the callbacks to the object states
-  POE::Session->create( object_states => [ $this => { _start => "start_state",
-                                                      _stop  => "stop_state",
+    # create the callbacks to the object states
+    POE::Session->create(
+        object_states => [
+            $this => {
+                _start => "start_state",
+                _stop  => "stop_state",
 
-                                                      irc_001    => "irc_001_state",
-                                                      irc_msg    => "irc_said_state",
-                                                      irc_public => "irc_said_state",
-                                                      irc_ctcp_action => "irc_emoted_state",
+                irc_001         => "irc_001_state",
+                irc_msg         => "irc_said_state",
+                irc_public      => "irc_said_state",
+                irc_ctcp_action => "irc_emoted_state",
+                irc_ping        => "irc_ping_state",
+                reconnect       => "reconnect",
 
-                                                      irc_disconnected => "irc_disconnected_state",
-                                                      irc_error        => "irc_error_state",
+                irc_disconnected => "irc_disconnected_state",
+                irc_error        => "irc_error_state",
 
-                                                      fork_close => "fork_close_state",
-                                                      fork_error => "fork_error_state"
-                                                    }
+                fork_close => "fork_close_state",
+                fork_error => "fork_error_state"
+            }
+        ]
+    );
 
-                                      ]);
+    # and say that we want to recive said messages
+    $poe_kernel->post( IRCNAME => register => 'all' );
 
-
-  # and say that we want to recive said messages
-  $poe_kernel->post( IRCNAME, 'regsiter', qw(msg public
-                                             join nick
-                                             connected disconnected error));
-
-  # run
-  $poe_kernel->run();
+    # run
+    $poe_kernel->run();
 }
 
 =item said($args)
@@ -190,8 +187,8 @@ sub said { undef }
 
 # default emoted will pass through to "said"
 sub emoted {
-  my ($self,$emoted_hashref) = @_;
-  $self->said($emoted_hashref);
+    my ( $self, $emoted_hashref ) = @_;
+    $self->said($emoted_hashref);
 }
 
 =item forkit
@@ -258,44 +255,52 @@ routine to pick them up and make sense of them.
 =cut
 
 sub forkit {
-  my $this = shift;
-  my $args = shift;
+    my $this = shift;
+    my $args;
+    if ( $#_ > 1 ) {
+        my %args = @_;
+        $args = \%args;
+    } else {
+        $args = shift;
+    }
 
-  return undef unless $args->{run};
+    return undef unless $args->{run};
 
-  $args->{handler}   = $args->{handler} || "fork_said";
-  $args->{arguments} = $args->{arguments} || [];
+    $args->{handler}   = $args->{handler}   || "fork_said";
+    $args->{arguments} = $args->{arguments} || [];
 
-  #install a new handler in the POE kernel pointing to
-  # $self->{$args{handler}}
-  $poe_kernel->state($args->{handler},$this);
+    #install a new handler in the POE kernel pointing to
+    # $self->{$args{handler}}
+    $poe_kernel->state( $args->{handler}, $this );
 
-  my $run;
-  if (ref($args->{run})=~/^CODE/) {
-    $run = sub { &{$args->{run}}($args->{body},@{$args->{arguments}}) };
-  } else {
-    $run = $args->{run};
-  }
+    my $run;
+    if ( ref( $args->{run} ) =~ /^CODE/ ) {
+        $run =
+          sub { &{ $args->{run} }( $args->{body}, @{ $args->{arguments} } ) };
+    } else {
+        $run = $args->{run};
+    }
 
-  my $wheel = POE::Wheel::Run->new
-      ( Program      => $run,
+    my $wheel = POE::Wheel::Run->new(
+        Program      => $run,
         StdoutFilter => POE::Filter::Line->new(),
         StderrFilter => POE::Filter::Line->new(),
         StdoutEvent  => "$args->{handler}",
         StderrEvent  => "fork_error",
         CloseEvent   => "fork_close"
-      );
+    );
 
-  # store the wheel object in our bot, so we can retrieve/delete easily
+    # store the wheel object in our bot, so we can retrieve/delete easily
 
-  $this->{forks}->{$wheel->ID} = {wheel   => $wheel,
-                                  args    => {
-                                              channel => $args->{channel},
-                                              who     => $args->{who},
-                                              address => $args->{address}
-                                             }
-                                 };
-  return undef;
+    $this->{forks}->{ $wheel->ID } = {
+        wheel => $wheel,
+        args  => {
+            channel => $args->{channel},
+            who     => $args->{who},
+            address => $args->{address}
+        }
+    };
+    return undef;
 }
 
 =item say
@@ -337,44 +342,44 @@ pass it on to a handler.
 
 =cut
 
-sub say
-{
-  # If we're called without an object ref, then we're handling saying
-  # stuff from inside a forked subroutine, so we'll freeze it, and toss
-  # it out on STDOUT so that POE::Wheel::Run's handler can pick it up.
-  if (!ref($_[0])) {
-    print $_[0]."\n";
-    return 1;
-  }
+sub say {
 
-  # Otherwise, this is a standard object method
+    # If we're called without an object ref, then we're handling saying
+    # stuff from inside a forked subroutine, so we'll freeze it, and toss
+    # it out on STDOUT so that POE::Wheel::Run's handler can pick it up.
+    if ( !ref( $_[0] ) ) {
+        print $_[0] . "\n";
+        return 1;
+    }
 
-  my $this = shift;
-  my $args;
-  if ($#_ > 1) {
-    my %args = @_;
-    $args = \%args;
-  } else {
-    $args = shift;
-  }
+    # Otherwise, this is a standard object method
 
-  my $body = $args->{body};
+    my $this = shift;
+    my $args;
+    if ( $#_ > 1 ) {
+        my %args = @_;
+        $args = \%args;
+    } else {
+        $args = shift;
+    }
 
-  # add the "Foo: bar" at the start
-  $body = "$args->{who}: $body"
-    if ($args->{channel} ne "msg" and $args->{address});
+    my $body = $args->{body};
 
-  # work out who we're going to send the message to
-  my $who = ($args->{channel} eq "msg") ? $args->{who} : $args->{channel};
+    # add the "Foo: bar" at the start
+    $body = "$args->{who}: $body"
+      if ( $args->{channel} ne "msg" and $args->{address} );
 
-  unless ($who && $body) {
-    print STDERR "Can't PRIVMSG without target and body\n";
-    print STDERR " who = '$who'\n body = '$body'\n";
-    return;
-  }
+    # work out who we're going to send the message to
+    my $who = ( $args->{channel} eq "msg" ) ? $args->{who} : $args->{channel};
 
-  # post an event that will send the message
-  $poe_kernel->post( IRCNAME, 'privmsg', $who, $body);
+    unless ( $who && $body ) {
+        print STDERR "Can't PRIVMSG without target and body\n";
+        print STDERR " who = '$who'\n body = '$body'\n";
+        return;
+    }
+
+    # post an event that will send the message
+    $poe_kernel->post( IRCNAME, 'privmsg', $who, $body );
 }
 
 =item emote
@@ -385,30 +390,40 @@ C<emote> will return data to channel, but emoted (as if you'd said
 =cut
 
 sub emote {
-  # If we're called without an object ref, then we're handling emoting
-  # stuff from inside a forked subroutine, so we'll freeze it, and
-  # toss it out on STDOUT so that POE::Wheel::Run's handler can pick
-  # it up.
-  if (!ref($_[0])) {
-    print $_[0]."\n";
-    return 1;
-  }
 
-  # Otherwise, this is a standard object method
+    # If we're called without an object ref, then we're handling emoting
+    # stuff from inside a forked subroutine, so we'll freeze it, and
+    # toss it out on STDOUT so that POE::Wheel::Run's handler can pick
+    # it up.
+    if ( !ref( $_[0] ) ) {
+        print $_[0] . "\n";
+        return 1;
+    }
 
-  my ($this,$args) = @_;
+    # Otherwise, this is a standard object method
 
-  my $body = $args->{body};
+    my $this = shift;
+    my $args;
+    if ( $#_ > 1 ) {
+        my %args = @_;
+        $args = \%args;
+    } else {
+        $args = shift;
+    }
 
-  # Work out who we're going to send the message to
-  my $who = ($args->{channel} eq "msg") ? $args->{who} :
-                                          $args->{channel};
+    my $body = $args->{body};
 
-  # post an event that will send the message
-  # if there's a better way of sending actions i'd love to know - jw
-  # me too; i'll look at it in v0.5 - sb
+    # Work out who we're going to send the message to
+    my $who =
+      ( $args->{channel} eq "msg" )
+      ? $args->{who}
+      : $args->{channel};
 
-  $poe_kernel->post(IRCNAME, 'privmsg', $who, "\cAACTION ".$body."\cA");
+    # post an event that will send the message
+    # if there's a better way of sending actions i'd love to know - jw
+    # me too; i'll look at it in v0.5 - sb
+
+    $poe_kernel->post( IRCNAME, 'privmsg', $who, "\cAACTION " . $body . "\cA" );
 }
 
 =item fork_said
@@ -423,14 +438,14 @@ nice object interface.
 =cut
 
 sub fork_said {
-  my ($this,$body,$wheel_id) = @_[0, ARG0, ARG1];
-  chomp($body); # remove newline necessary to move data;
+    my ( $this, $body, $wheel_id ) = @_[ 0, ARG0, ARG1 ];
+    chomp($body);    # remove newline necessary to move data;
 
-  # pick up the default arguments we squirreled away earlier
-  my $args = $this->{forks}->{$wheel_id}->{args};
-  $args->{body}=$body;
+    # pick up the default arguments we squirreled away earlier
+    my $args = $this->{forks}->{$wheel_id}->{args};
+    $args->{body} = $body;
 
-  $this->say($args);
+    $this->say($args);
 }
 
 =item help
@@ -472,11 +487,10 @@ The server we're going to connect to.  Defaults to
 
 =cut
 
-sub server
-{
-  my $this = shift;
-  $this->{server} = shift if @_;
-  return $this->{server} || "london.rhizomatic.net"
+sub server {
+    my $this = shift;
+    $this->{server} = shift if @_;
+    return $this->{server} || "london.rhizomatic.net";
 }
 
 =item port
@@ -485,11 +499,10 @@ The port we're going to use.  Defaults to "6667"
 
 =cut
 
-sub port
-{
-  my $this = shift;
-  $this->{port} = shift if @_;
-  return $this->{port} || "6667"
+sub port {
+    my $this = shift;
+    $this->{port} = shift if @_;
+    return $this->{port} || "6667";
 }
 
 =item nick
@@ -499,17 +512,15 @@ and numbers followed by the word "bot"
 
 =cut
 
-sub nick
-{
-  my $this = shift;
-  $this->{nick} = shift if @_;
-  return $this->{nick} ||= _random_nick();
+sub nick {
+    my $this = shift;
+    $this->{nick} = shift if @_;
+    return $this->{nick} ||= _random_nick();
 }
 
-sub _random_nick
-{
-  my @things = ('a'..'z');
-  return join '', (map { @things[rand @things] } 0..4), "bot";
+sub _random_nick {
+    my @things = ( 'a' .. 'z' );
+    return join '', ( map { @things[ rand @things ] } 0 .. 4 ), "bot";
 }
 
 =item alt_nicks
@@ -523,16 +534,15 @@ even though it isn't really.
 
 =cut
 
-sub alt_nicks
-{
-  my $this = shift;
-  if (@_)
-  {
-    # make sure we copy
-    my @args = (ref $_[0] eq "ARRAY") ? @{$_[0]} : @_;
-    $this->{alt_nicks} = \@args;
-  }
-  @{ $this->{alt_nicks} || [] }
+sub alt_nicks {
+    my $this = shift;
+    if (@_) {
+
+        # make sure we copy
+        my @args = ( ref $_[0] eq "ARRAY" ) ? @{ $_[0] } : @_;
+        $this->{alt_nicks} = \@args;
+    }
+    @{ $this->{alt_nicks} || [] };
 }
 
 =item username
@@ -542,11 +552,10 @@ will be the same as our nick.
 
 =cut
 
-sub username
-{
-  my $this = shift;
-  $this->{username} = shift if @_;
-  $this->{username} or $this->nick
+sub username {
+    my $this = shift;
+    $this->{username} = shift if @_;
+    $this->{username} or $this->nick;
 }
 
 =item name
@@ -556,11 +565,10 @@ The name that the bot will identify itself as.  Defaults to
 
 =cut
 
-sub name
-{
-  my $this = shift;
-  $this->{name} = shift if @_;
-  $_[0]->{name} or $this->nick . " bot"
+sub name {
+    my $this = shift;
+    $this->{name} = shift if @_;
+    $_[0]->{name} or $this->nick . " bot";
 }
 
 =item channels
@@ -569,16 +577,15 @@ The channels we're going to connect to.
 
 =cut
 
-sub channels
-{
-  my $this = shift;
-  if (@_)
-  {
-    # make sure we copy
-    my @args = (ref $_[0] eq "ARRAY") ? @{$_[0]} : @_;
-    $this->{channels} = \@args;
-  }
-  @{ $this->{channels} || [] }
+sub channels {
+    my $this = shift;
+    if (@_) {
+
+        # make sure we copy
+        my @args = ( ref $_[0] eq "ARRAY" ) ? @{ $_[0] } : @_;
+        $this->{channels} = \@args;
+    }
+    @{ $this->{channels} || [] };
 }
 
 =item quit_message
@@ -587,11 +594,10 @@ The quit message.  Defaults to "Bye".
 
 =cut
 
-sub quit_message
-{
-  my $this = shift;
-  $this->{quit_message} = shift if @_;
-  defined ($this->{quit_message}) ? $this->{quit_message} : "Bye";
+sub quit_message {
+    my $this = shift;
+    $this->{quit_message} = shift if @_;
+    defined( $this->{quit_message} ) ? $this->{quit_message} : "Bye";
 }
 
 =item ignore_list
@@ -601,16 +607,15 @@ other bots.)  Useful for stopping bot cascades.
 
 =cut
 
-sub ignore_list
-{
-  my $this = shift;
-  if (@_)
-  {
-    # make sure we copy
-    my @args = (ref $_[0] eq "ARRAY") ? @{$_[0]} : @_;
-    $this->{ignore_list} = \@args;
-  }
-  @{ $this->{ignore_list} || []}
+sub ignore_list {
+    my $this = shift;
+    if (@_) {
+
+        # make sure we copy
+        my @args = ( ref $_[0] eq "ARRAY" ) ? @{ $_[0] } : @_;
+        $this->{ignore_list} = \@args;
+    }
+    @{ $this->{ignore_list} || [] };
 }
 
 =head2 States
@@ -627,50 +632,54 @@ Called when we start.  Used to fire a "connect to irc server event"
 =cut
 
 sub start_state {
-  my ($this, $kernel, $session) = @_[OBJECT, KERNEL, SESSION];
+    my ( $this, $kernel, $session ) = @_[ OBJECT, KERNEL, SESSION ];
 
-  $this->log("Control session start\n");
+    $this->log("Control session start\n");
 
-  # Make an alias for our session, to keep it from getting GC'ed.
-  $kernel->alias_set( ALIASNAME );
+    # Make an alias for our session, to keep it from getting GC'ed.
+    $kernel->alias_set(ALIASNAME);
 
-  # Ask the IRC component to send us all IRC events it receives. This
-  # is the easy, indiscriminate way to do it.
-  $kernel->post( IRCNAME, 'register', 'all');
+    # Ask the IRC component to send us all IRC events it receives. This
+    # is the easy, indiscriminate way to do it.
+    $kernel->post(IRCNAME, 'register', 'all');
 
-  # Setting Debug to 1 causes P::C::IRC to print all raw lines of text
-  # sent to and received from the IRC server. Very useful for debugging.
-  $kernel->post( IRCNAME, 'connect', { Debug    => 0,
-                                       Nick     => $this->nick,
-                                       Server   => $this->server,
-                                       Port     => $this->port,
-                                       Username => $this->username,
-                                       Ircname  => $this->name, }
-                           );
-  $kernel->delay( 'reconnect', 20);
+    # Setting Debug to 1 causes P::C::IRC to print all raw lines of text
+    # sent to and received from the IRC server. Very useful for debugging.
+    $kernel->post(IRCNAME,'connect',
+        {
+            Debug    => 0,
+            Nick     => $this->nick,
+            Server   => $this->server,
+            Port     => $this->port,
+            Username => $this->username,
+            Ircname  => $this->name,
+        }
+    );
+    $kernel->delay('reconnect', 500);
 
 }
 
-=item reconnect_state
+sub reconnect {
+    my ( $this, $kernel, $session ) = @_[ OBJECT, KERNEL, SESSION ];
 
-Called in order to try aggressive reconnecting. A little overhead
-here, because the bot just blindly tests whether it's connected every
-20 seconds, but it means that server drops will be picked up, and the
-bot will be far stabler during netsplits and random sever barfs.
+    $this->log("I think I've lost the server. restarting..\n");
 
-=cut
+    $kernel->call( IRCNAME, 'disconnect' );
+    $kernel->call( IRCNAME, 'shutdown' );
+    POE::Component::IRC->new(IRCNAME);
+    $kernel->post( IRCNAME, 'register', 'all' );
 
-sub reconnect_state {
-  my ($this, $kernel, $session) = @_[OBJECT, KERNEL, SESSION];
-
-  $kernel->post( IRCNAME, 'connect', { Debug    => 0,
-                                       Nick     => $this->nick,
-                                       Server   => $this->server,
-                                       Port     => $this->port,
-                                       Username => $this->username,
-                                       Ircname  => $this->name, }
-                           );
-  $kernel->delay( 'reconnect', 20);
+    $kernel->post(IRCNAME, 'connect',
+        {
+            Debug    => 0,
+            Nick     => $this->nick,
+            Server   => $this->server,
+            Port     => $this->port,
+            Username => $this->username,
+            Ircname  => $this->name,
+        }
+    );
+    $kernel->delay('reconnect', 500);
 }
 
 =item stop_state
@@ -679,14 +688,13 @@ Called when we're stopping.  Shutdown the bot correctly.
 
 =cut
 
-sub stop_state
-{
-  my ($this, $kernel) = @_[OBJECT, KERNEL];
+sub stop_state {
+    my ( $this, $kernel ) = @_[ OBJECT, KERNEL ];
 
-  $this->log("Control session stopped.\n");
+    $this->log("Control session stopped.\n");
 
-  $kernel->post( IRCNAME, 'quit', $this->quit_message );
-  $kernel->alias_remove( ALIASNAME );
+    $kernel->post( IRCNAME, 'quit', $this->quit_message );
+    $kernel->alias_remove(ALIASNAME);
 }
 
 =item irc_001_state
@@ -698,23 +706,21 @@ We also ignore ourselves.  We don't want to hear what we have to say.
 
 =cut
 
-sub irc_001_state
-{
-  my ($this, $kernel) = @_[OBJECT, KERNEL];
+sub irc_001_state {
+    my ( $this, $kernel ) = @_[ OBJECT, KERNEL ];
 
-  $this->log("IRC server ready\n");
+    $this->log("IRC server ready\n");
 
-  # ignore all messages from ourselves
-  $kernel->post( IRCNAME, 'mode', $this->nick, '+i' );
+    # ignore all messages from ourselves
+    $kernel->post( IRCNAME, 'mode', $this->nick, '+i' );
 
-  # connect to the channel
-  foreach my $channel ($this->channels)
-  {
-    $this->log("Trying to connect to '$channel'\n");
-    $kernel->post( IRCNAME , 'join', $channel );
-  }
+    # connect to the channel
+    foreach my $channel ( $this->channels ) {
+        $this->log("Trying to connect to '$channel'\n");
+        $kernel->post( IRCNAME, 'join', $channel );
+    }
 
-  $this->connected();
+    $this->connected();
 }
 
 =item irc_disconnected_state
@@ -724,11 +730,11 @@ then dies.
 
 =cut
 
-sub irc_disconnected_state
-{
-  my ($this, $server) = @_[OBJECT, ARG0];
-  $this->log("Lost connection to server $server.\n");
-  die "IRC Disconnect: $server";
+sub irc_disconnected_state {
+    my ( $this, $server ) = @_[ OBJECT, ARG0 ];
+    $this->log("Lost connection to server $server.\n");
+
+    #  die "IRC Disconnect: $server";
 }
 
 =item irc_error_state
@@ -737,11 +743,11 @@ Called if there is an irc server error.  Logs the error and then dies.
 
 =cut
 
-sub irc_error_state
-{
-  my ($this, $err) = @_[OBJECT, ARG0];
-  $this->log("Server error occurred! $err\n");
-  die "IRC Error: $err";
+sub irc_error_state {
+    my ( $this, $err ) = @_[ OBJECT, ARG0 ];
+    $this->log("Server error occurred! $err\n");
+
+    #  die "IRC Error: $err";
 }
 
 =item irc_kicked_state
@@ -753,9 +759,8 @@ wanted, the best thing to do would be to hang around off channel.
 
 =cut
 
-sub irc_kicked_state
-{
-  my ($this, $err) = @_[OBJECT, ARG0];
+sub irc_kicked_state {
+    my ( $this, $err ) = @_[ OBJECT, ARG0 ];
 }
 
 =item irc_join_state
@@ -764,9 +769,8 @@ Called if someone joins.  Used for nick tracking
 
 =cut
 
-sub irc_join_state
-{
-  my ($this, $nick) = @_[OBJECT, ARG0];
+sub irc_join_state {
+    my ( $this, $nick ) = @_[ OBJECT, ARG0 ];
 }
 
 =item irc_nick_state
@@ -775,9 +779,8 @@ Called if someone changes nick.  Used for nick tracking
 
 =cut
 
-sub irc_nick_state
-{
-  my ($this, $nick, $newnick) = @_[OBJECT, ARG0, ARG1];
+sub irc_nick_state {
+    my ( $this, $nick, $newnick ) = @_[ OBJECT, ARG0, ARG1 ];
 }
 
 =item irc_said_state
@@ -788,7 +791,7 @@ formats it into a nicer format and calls 'said'
 =cut
 
 sub irc_said_state {
-  irc_received_state('said','say',@_);
+    irc_received_state( 'said', 'say', @_ );
 }
 
 =item irc_emoted_state
@@ -800,7 +803,7 @@ which deals with it as if it was a spoken phrase.
 =cut
 
 sub irc_emoted_state {
-  irc_received_state('emoted','emote',@_);
+    irc_received_state( 'emoted', 'emote', @_ );
 }
 
 =item irc_received_state
@@ -811,88 +814,95 @@ channel input into a more copable-with format.
 =cut
 
 sub irc_received_state {
-  my $received = shift;
-  my $respond = shift;
-  my $return;
+    my $received = shift;
+    my $respond  = shift;
+    my ( $this, $nick, $to, $body ) = @_[ OBJECT, ARG0, ARG1, ARG2 ];
 
-  my ($this, $nick, $to, $body) = @_[OBJECT, ARG0, ARG1, ARG2];
+    my $return;
 
-  my $mess = {};
+    my $mess = {};
 
-  # work out who it was from
-  $mess->{who} = $this->nick_strip($nick);
+    # work out who it was from
+    $mess->{who} = $this->nick_strip($nick);
 
-  # right, get the list of places this message was
-  # sent to and work out the first one that we're
-  # either a memeber of is is our nick.
-  # The IRC protocol allows messages to be sent to multiple
-  # targets, which is pretty clever. However, noone actually
-  # /does/ this, so we can get away with this:
+    # right, get the list of places this message was
+    # sent to and work out the first one that we're
+    # either a memeber of is is our nick.
+    # The IRC protocol allows messages to be sent to multiple
+    # targets, which is pretty clever. However, noone actually
+    # /does/ this, so we can get away with this:
 
-  my $channel = $to->[0];
-  if ($channel eq $this->nick)
-  {
-    $mess->{channel} = "msg";
-    $mess->{address} = "msg";
-  }
-  else {
-    $mess->{channel} = $channel;
-  }
-
-  # okay, work out if we're addressed or not
-
-  $mess->{body} = $body;
-  unless ($mess->{channel} eq "msg")
-  {
-    my $nick = $this->nick;
-    ($mess->{address}) = $mess->{body} =~ s/^(\Q$nick\E\s*[:,-]?)//;
-
-    foreach $nick ($this->alt_nicks)
-    {
-      last if $this->{address};
-
-      ($mess->{address}) = $mess->{body} =~ s/^(\Q$nick\E\s*[:,-])//;
+    my $channel = $to->[0];
+    if ( $channel eq $this->nick ) {
+        $mess->{channel} = "msg";
+        $mess->{address} = "msg";
+    } else {
+        $mess->{channel} = $channel;
     }
-  }
 
-  # strip off whitespace before and after the message
-  $mess->{body} =~ s/^\s+//;
-  $mess->{body} =~ s/\s+$//;
+    # okay, work out if we're addressed or not
 
-  # okay, we got this far.  Better log this.  This needs changing
-  # to a nice format.  Oooh, I could spit out sax events... (mf)
-  $this->log((join '||', $mess->{who},
-                         $mess->{channel},
-                         $mess->{address},
-                         $mess->{body})."\n");
+    $mess->{body} = $body;
+    unless ( $mess->{channel} eq "msg" ) {
+        my $nick = $this->nick;
+        ( $mess->{address} ) = $mess->{body} =~ s/^(\Q$nick\E\s*[:,-]?)//;
 
-  # check if someone was asking for help
-  if ($mess->{address} && ($mess->{body} =~ /^help/i))
-  {
-    $this->log("Invoking help for '$mess->{who}'\n");
-    $mess->{body} = $this->help($mess);
-    $this->say($mess);
-    return;
-  }
+        foreach $nick ( $this->alt_nicks ) {
+            last if $this->{address};
 
-  # okay, call the said/emoted method
-  my $respond = $this->$received($mess);
+            ( $mess->{address} ) = $mess->{body} =~ s/^(\Q$nick\E\s*[:,-])//;
+        }
+    }
 
-  ### what did we get back?
+    # strip off whitespace before and after the message
+    $mess->{body} =~ s/^\s+//;
+    $mess->{body} =~ s/\s+$//;
 
-  # nothing? Say nothing then
-  return unless defined($return);
+    # okay, we got this far.  Better log this.  This needs changing
+    # to a nice format.  Oooh, I could spit out sax events... (mf)
+    $this->log(
+        join('||', $mess->{who}, $mess->{channel}, $mess->{address}, $mess->{body})
+        ."\n");
 
-  # a string?  Say it how we were addressed then
-  unless (ref($return))
-  {
-    $mess->{body} = $return;
-    $this->$respond($mess);
-    return;
-  }
+    # check if someone was asking for help
+    if ( $mess->{address} && ( $mess->{body} =~ /^help/i ) ) {
+        $this->log("Invoking help for '$mess->{who}'\n");
+        $mess->{body} = $this->help($mess);
+        $this->say($mess);
+        return;
+    }
 
-  # just say what we were handed back
-  $this->$respond($return);
+    # okay, call the said/emoted method
+    $respond = $this->$received($mess);
+
+    ### what did we get back?
+
+    # nothing? Say nothing then
+    return unless defined($return);
+
+    # a string?  Say it how we were addressed then
+    unless ( ref($return) ) {
+        $mess->{body} = $return;
+        $this->$respond($mess);
+        return;
+    }
+
+    # just say what we were handed back
+    $this->$respond($return);
+}
+
+=item irc_ping_state
+
+The most reliable way I've found of doing auto-server-rejoin is to listen for
+pings. Every ping we get, we put off rejoining the server for another few mins.
+If we haven't heard a ping in a while, the rejoin code will get called.
+
+=cut
+
+sub irc_ping_state {
+    my ( $this, $kernel, $heap ) = @_[ OBJECT, KERNEL, HEAP ];
+    $this->log("PING\n");
+    $kernel->delay( 'reconnect', 500 );
 }
 
 =item fork_close_state
@@ -904,7 +914,8 @@ from memory.
 =cut
 
 sub fork_close_state {
-    my ($this,$wheel_id)=@_[0,ARG0];
+    my ( $this, $wheel_id ) = @_[ 0, ARG0 ];
+
     #warn "received close event from wheel $wheel_id\n";
     delete $this->{forks}->{$wheel_id};
 }
@@ -917,14 +928,12 @@ in derived classes to be more useful
 
 =cut
 
-sub fork_error_state {}
+sub fork_error_state { }
 
 =back
 
 =head2 Other States
 
-=head2 Other States
-
 Bot::BasicBot implements AUTOLOAD for sending arbitrary states to the
 underlying POE::Component::IRC compoment. So for a $bot object, sending
 
@@ -936,12 +945,11 @@ is equivalent to
 
 =cut
 
-sub AUTOLOAD
-{
-  my $this = shift;
-  our $AUTOLOAD;
-  $AUTOLOAD =~ s/.*:://;
-  $poe_kernel->post(IRCNAME, $AUTOLOAD, @_);
+sub AUTOLOAD {
+    my $this = shift;
+    our $AUTOLOAD;
+    $AUTOLOAD =~ s/.*:://;
+    $poe_kernel->post( IRCNAME, $AUTOLOAD, @_ );
 }
 
 =head2 Methods
@@ -955,19 +963,15 @@ otherwise simple prints the message to STDERR.
 
 =cut
 
-sub log
-{
-  my $this = shift;
-  my $text = shift;
+sub log {
+    my $this = shift;
+    my $text = shift;
 
-  if ($this->{log})
-  {
-    $this->{log}->log(time . " $text");
-  }
-  else
-  {
-    print STDERR $text;
-  }
+    if ( $this->{log} ) {
+        $this->{log}->log( time . " $text" );
+    } else {
+        print STDERR $text;
+    }
 }
 
 =item get($key) or get($storename, $key)
@@ -978,24 +982,18 @@ uses the "main" store.
 
 =cut
 
-sub get
-{
-  my $this      = shift;
-  my $key       = pop;
-  my $storename = pop || "main";
+sub get {
+    my $this      = shift;
+    my $key       = pop;
+    my $storename = pop || "main";
 
-  if ($this->{store})
-  {
-    return $this->{store}->get($storename,$key)
-  }
-  elsif ($this->{tempstore}{ $storename })
-  {
-    $this->{tempstore}{ $storename }{ $key };
-  }
-  else
-  {
-    return undef;
-  }
+    if ( $this->{store} ) {
+        return $this->{store}->get( $storename, $key );
+    } elsif ( $this->{tempstore}{$storename} ) {
+        $this->{tempstore}{$storename}{$key};
+    } else {
+        return undef;
+    }
 }
 
 =item set($key, $value) or set($storename, $key, $value)
@@ -1005,22 +1003,20 @@ initilised, otherwise uses an internal hash.
 
 =cut
 
-sub set
-{
-  my $this      = shift;
-  my $value     = pop;
-  my $key       = pop;
-  my $storename = pop || "main";
+sub set {
+    my $this      = shift;
+    my $value     = pop;
+    my $key       = pop;
+    my $storename = pop || "main";
 
-  if ($this->{store})
-  {
-    return $this->{store}->get($storename,$key)
-  }
+    if ( $this->{store} ) {
+        return $this->{store}->get( $storename, $key );
+    }
 
-  $this->{tempstore}{ $storename } = {}
-    unless ($this->{tempstore}{ $storename });
+    $this->{tempstore}{$storename} = {}
+      unless ( $this->{tempstore}{$storename} );
 
-  $this->{tempstore}{ $storename }{ $key } = $value;
+    $this->{tempstore}{$storename}{$key} = $value;
 }
 
 =item ignore_nick($nick)
@@ -1030,15 +1026,11 @@ the ignore list or with a nick ending in "bot".
 
 =cut
 
-sub ignore_nick
-{
-  local $_;
-
-  my $this = shift;
-  my $nick = shift;
-
-  return 1 if $nick =~ /bot$/;
-  return grep {$nick eq $_} @{ $this->{ignore_list} };
+sub ignore_nick {
+    local $_;
+    my $this = shift;
+    my $nick = shift;
+    return grep { $nick eq $_ } @{ $this->{ignore_list} };
 }
 
 =item nick_strip
@@ -1048,20 +1040,19 @@ returns just the nick
 
 =cut
 
-sub nick_strip
-{
-  my $this = shift;
-  my $combined = shift;
-  my ($nick) = $combined =~ m/(.*?)!/;
+sub nick_strip {
+    my $this     = shift;
+    my $combined = shift;
+    my ($nick) = $combined =~ m/(.*?)!/;
 
-  return $nick;
+    return $nick;
 }
 
 =back
 
 =head1 AUTHOR
 
-Simon Batistoni E<lt>simon@hitherto.netE<gt>
+Tom Insam E<lt>tom@jerakeen.orgE<gt>
 
 This program is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
@@ -1076,6 +1067,9 @@ Nice code for dealing with emotes thanks to Jo Walsh.
 Various patches from Tom Insam, including much improved rejoining,
 AUTOLOAD stuff, better interactive help, and a few API tidies.
 
+Maintainership for a while was in the hands of Simon Kent
+E<lt>simon@hitherto.netE<gt>. Don't know what he did. :-)
+
 =head1 SYSTEM REQUIREMENTS
 
 Bot::BasicBot is based on POE, and really needs the latest version as
